@@ -45,6 +45,17 @@ void HeaderGenerator::writeCopyCtor(QTextStream& s, const AbstractMetaClass* met
     s << INDENT << "}" << endl << endl;
 }
 
+void HeaderGenerator::writeProtectedFieldAccessors(QTextStream& s, const AbstractMetaField* field) const
+{
+    QString fieldType = field->type()->cppSignature();
+    QString fieldName = field->enclosingClass()->qualifiedCppName() + "::" + field->name();
+
+    s << INDENT << "inline " << fieldType << ' ' << protectedFieldGetterName(field) << "()";
+    s << " { return " << fieldName << "; }" << endl;
+    s << INDENT << "inline void " << protectedFieldSetterName(field) << '(' << fieldType << " value)";
+    s << " { " << fieldName << " = value; }" << endl;
+}
+
 void HeaderGenerator::generateClass(QTextStream& s, const AbstractMetaClass* metaClass)
 {
     ReportHandler::debugSparse("Generating header for " + metaClass->fullName());
@@ -83,14 +94,35 @@ void HeaderGenerator::generateClass(QTextStream& s, const AbstractMetaClass* met
         if (isCopyable(metaClass))
             writeCopyCtor(s, metaClass);
 
-        foreach (AbstractMetaFunction *func, filterFunctions(metaClass))
+        bool hasVirtualFunction = false;
+        foreach (AbstractMetaFunction *func, filterFunctions(metaClass)) {
+            if (func->isVirtual())
+                hasVirtualFunction = true;
             writeFunction(s, func);
+        }
+
+#ifdef AVOID_PROTECTED_HACK
+        if (metaClass->hasProtectedFields()) {
+            foreach (AbstractMetaField* field, metaClass->fields()) {
+                if (!field->isProtected())
+                    continue;
+                writeProtectedFieldAccessors(s, field);
+            }
+        }
+#endif
 
         //destructor
-        s << INDENT << (metaClass->hasVirtualDestructor() ? "virtual " : "") << "~" << wrapperName << "();" << endl;
+#ifdef AVOID_PROTECTED_HACK
+        if (!metaClass->hasPrivateDestructor())
+#endif
+            s << INDENT << (metaClass->hasVirtualDestructor() || hasVirtualFunction ? "virtual " : "") << "~" << wrapperName << "();" << endl;
 
         writeCodeSnips(s, metaClass->typeEntry()->codeSnips(), CodeSnip::Declaration, TypeSystem::NativeCode);
 
+#ifdef AVOID_PROTECTED_HACK
+        if (!metaClass->hasPrivateDestructor()) {
+#endif
+
         if (usePySideExtensions() && metaClass->isQObject()) {
             s << "public:\n";
             s << INDENT << "virtual int qt_metacall(QMetaObject::Call call, int id, void** args);\n";
@@ -98,6 +130,10 @@ void HeaderGenerator::generateClass(QTextStream& s, const AbstractMetaClass* met
             s << INDENT << "mutable PySide::DynamicQMetaObject* m_metaObject;\n";
         }
 
+#ifdef AVOID_PROTECTED_HACK
+        }
+#endif
+
         s << "};" << endl << endl;
     }
 
@@ -114,11 +150,26 @@ void HeaderGenerator::writeFunction(QTextStream& s, const AbstractMetaFunction*
         return;
 
 #ifdef AVOID_PROTECTED_HACK
-    if (func->isProtected() && !func->isConstructor()) {
+    if (func->isProtected() && !func->isConstructor() && !func->isOperatorOverload()) {
         s << INDENT << "inline " << (func->isStatic() ? "static " : "");
-        s << functionSignature(func, "", "_protected") << " { ";
-        s << (func->type() ? "return " : "") << func->ownerClass()->qualifiedCppName() << "::";
-        writeFunctionCall(s, func);
+        s << functionSignature(func, "", "_protected", Generator::EnumAsInts|Generator::OriginalTypeDescription) << " { ";
+        s << (func->type() ? "return " : "");
+        if (!func->isAbstract())
+            s << func->ownerClass()->qualifiedCppName() << "::";
+        s << func->originalName() << '(';
+        QStringList args;
+        foreach (const AbstractMetaArgument* arg, func->arguments()) {
+            QString argName = arg->argumentName();
+            const TypeEntry* enumTypeEntry = 0;
+            if (arg->type()->isFlags())
+                enumTypeEntry = reinterpret_cast<const FlagsTypeEntry*>(arg->type()->typeEntry())->originator();
+            else if (arg->type()->isEnum())
+                enumTypeEntry = arg->type()->typeEntry();
+            if (enumTypeEntry)
+                argName = QString("%1(%2)").arg(arg->type()->cppSignature()).arg(argName);
+            args << argName;
+        }
+        s << args.join(", ") << ')';
         s << "; }" << endl;
     }
 #endif
@@ -127,6 +178,11 @@ void HeaderGenerator::writeFunction(QTextStream& s, const AbstractMetaFunction*
     if (func->isPrivate() || (func->isModifiedRemoved() && !func->isAbstract()))
         return;
 
+#ifdef AVOID_PROTECTED_HACK
+    if (func->ownerClass()->hasPrivateDestructor() && (func->isAbstract() || func->isVirtual()))
+        return;
+#endif
+
     if (func->isConstructor() || func->isAbstract() || func->isVirtual()) {
         s << INDENT;
         Options virtualOption = Generator::NoOption;
@@ -165,6 +221,15 @@ void HeaderGenerator::writeTypeConverterDecl(QTextStream& s, const TypeEntry* ty
     bool isValueTypeWithImplConversions = type->isValue() && !implicitConvs.isEmpty();
     bool hasCustomConversion = type->hasConversionRule();
     QString typeT = type->name() + (isAbstractOrObjectType ? "*" : "");
+    QString typeName = type->name();
+
+#ifdef AVOID_PROTECTED_HACK
+    const AbstractMetaEnum* metaEnum = findAbstractMetaEnum(type);
+    if (metaEnum && metaEnum->isProtected()) {
+        typeT = protectedEnumSurrogateName(metaEnum);
+        typeName = typeT;
+    }
+#endif
 
     s << "struct Converter<" << typeT << " >";
     if (!hasCustomConversion) {
@@ -174,7 +239,7 @@ void HeaderGenerator::writeTypeConverterDecl(QTextStream& s, const TypeEntry* ty
             s << " : ObjectTypeConverter";
         else
             s << " : ValueTypeConverter";
-        s << '<' << type->name() << " >";
+        s << '<' << typeName << " >";
     }
     s << endl << '{' << endl;
     if (isValueTypeWithImplConversions || hasCustomConversion) {
@@ -394,11 +459,18 @@ void HeaderGenerator::writeExportMacros(QTextStream& s)
 
 void HeaderGenerator::writeSbkTypeFunction(QTextStream& s, const AbstractMetaEnum* cppEnum)
 {
-    QString enumPrefix;
+    QString enumName = cppEnum->name();
     if (cppEnum->enclosingClass())
-        enumPrefix = cppEnum->enclosingClass()->qualifiedCppName() + "::";
-    s <<  "template<> inline PyTypeObject* SbkType<" << enumPrefix << cppEnum->name() << " >() "
-      << "{ return " << cpythonTypeNameExt(cppEnum->typeEntry()) << "; }\n";
+        enumName = cppEnum->enclosingClass()->qualifiedCppName() + "::" + enumName;
+#ifdef AVOID_PROTECTED_HACK
+    if (cppEnum->isProtected()) {
+        enumName = protectedEnumSurrogateName(cppEnum);
+        s << "enum " << enumName << " {};" << endl;
+    }
+#endif
+
+    s << "template<> inline PyTypeObject* SbkType<" << enumName << " >() ";
+    s << "{ return " << cpythonTypeNameExt(cppEnum->typeEntry()) << "; }\n";
 
     FlagsTypeEntry* flag = cppEnum->typeEntry()->flags();
     if (flag) {
