@@ -156,12 +156,20 @@ QString ShibokenGenerator::translateTypeForWrapperMethod(const AbstractMetaType*
                                                          const AbstractMetaClass* context) const
 {
     QString result;
-    const TypeEntry* tentry = cType->typeEntry();
 
-    if (cType->isArray())
+    if (cType->isArray()) {
         result = translateTypeForWrapperMethod(cType->arrayElementType(), context) + "[]";
-    else
-        result = translateType(cType, context);
+    } else {
+#ifdef AVOID_PROTECTED_HACK
+        if (cType->isEnum()) {
+            const AbstractMetaEnum* metaEnum = findAbstractMetaEnum(cType);
+            if (metaEnum && metaEnum->isProtected())
+                result = protectedEnumSurrogateName(metaEnum);
+        }
+        if (result.isEmpty())
+#endif
+            result = translateType(cType, context);
+    }
 
     return result;
 }
@@ -170,9 +178,24 @@ bool ShibokenGenerator::shouldGenerateCppWrapper(const AbstractMetaClass* metaCl
 {
     bool result = metaClass->isPolymorphic() || metaClass->hasVirtualDestructor();
 #ifdef AVOID_PROTECTED_HACK
-    result = result || metaClass->hasProtectedFunctions() || metaClass->hasProtectedDestructor();
+    result = result || metaClass->hasProtectedFields() || metaClass->hasProtectedDestructor();
+    if (!result && metaClass->hasProtectedFunctions()) {
+        int protectedFunctions = 0;
+        int protectedOperators = 0;
+        foreach (const AbstractMetaFunction* func, metaClass->functions()) {
+            if (!func->isProtected() || func->isSignal() || func->isModifiedRemoved())
+                continue;
+            else if (func->isOperatorOverload())
+                protectedOperators++;
+            else
+                protectedFunctions++;
+        }
+        result = result || (protectedFunctions > protectedOperators);
+    }
+#else
+    result = result && !metaClass->hasPrivateDestructor();
 #endif
-    return result && !metaClass->isNamespace() && !metaClass->hasPrivateDestructor();
+    return result && !metaClass->isNamespace();
 }
 
 QString ShibokenGenerator::wrapperName(const AbstractMetaClass* metaClass)
@@ -189,6 +212,21 @@ QString ShibokenGenerator::wrapperName(const AbstractMetaClass* metaClass)
     }
 }
 
+QString ShibokenGenerator::protectedEnumSurrogateName(const AbstractMetaEnum* metaEnum)
+{
+    return metaEnum->fullName().replace(".", "_") + "_Surrogate";
+}
+
+QString ShibokenGenerator::protectedFieldGetterName(const AbstractMetaField* field)
+{
+    return QString("protected_%1_getter").arg(field->name());
+}
+
+QString ShibokenGenerator::protectedFieldSetterName(const AbstractMetaField* field)
+{
+    return QString("protected_%1_setter").arg(field->name());
+}
+
 QString ShibokenGenerator::cpythonFunctionName(const AbstractMetaFunction* func)
 {
     QString result;
@@ -300,6 +338,13 @@ void ShibokenGenerator::writeBaseConversion(QTextStream& s, const TypeEntry* typ
     QString typeName = type->name();
     if (type->isObject())
         typeName.append('*');
+#ifdef AVOID_PROTECTED_HACK
+    if (type->isEnum()) {
+        const AbstractMetaEnum* metaEnum = findAbstractMetaEnum(type);
+        if (metaEnum && metaEnum->isProtected())
+            typeName = protectedEnumSurrogateName(metaEnum);
+    }
+#endif
     s << baseConversionString(typeName);
 }
 
