@@ -24,9 +24,9 @@
 #ifndef SHIBOKENGENERATOR_H
 #define SHIBOKENGENERATOR_H
 
-#define PYTHON_RETURN_VAR         "py_result"
-#define CPP_RETURN_VAR            "cpp_result"
-#define THREAD_STATE_SAVER_VAR    "thread_state_saver"
+#define PYTHON_RETURN_VAR         "pyResult"
+#define CPP_RETURN_VAR            "cppResult"
+#define THREAD_STATE_SAVER_VAR    "threadStateSaver"
 #define CPP_SELF_VAR              "cppSelf"
 
 #include <generator.h>
@@ -213,6 +213,10 @@ public:
 
     static QString wrapperName(const AbstractMetaClass* metaClass);
 
+    static QString protectedEnumSurrogateName(const AbstractMetaEnum* metaEnum);
+    static QString protectedFieldGetterName(const AbstractMetaField* field);
+    static QString protectedFieldSetterName(const AbstractMetaField* field);
+
     static QString pythonPrimitiveTypeName(const QString& cppTypeName);
     static QString pythonPrimitiveTypeName(const PrimitiveTypeEntry* type);
 
@@ -334,7 +338,7 @@ protected:
                            const AbstractMetaFunction* metaFunc,
                            Options options = NoOption) const;
 
-    AbstractMetaFunctionList filterFunctions(const AbstractMetaClass* metaClass);
+    static AbstractMetaFunctionList filterFunctions(const AbstractMetaClass* metaClass);
 
     // All data about extended converters: the type entries of the target type, and a
     // list of AbstractMetaClasses accepted as argument for the conversion.
